@@ -2,12 +2,17 @@
 suppressWarnings(RNGversion(vstr = "3.5.3"))
 set.seed(seed = 42)
 
-# Tests for FindMarkers default parameters
+# Tests for FindMarkers
 # --------------------------------------------------------------------------------
 context("FindMarkers")
 
+clr.obj <- suppressWarnings(NormalizeData(pbmc_small, normalization.method = "CLR"))
+sct.obj <- suppressWarnings(suppressMessages(SCTransform(pbmc_small)))
+
 markers.0 <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, verbose = FALSE, base = exp(1)))
 markers.01 <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1)))
+results.clr <- suppressWarnings(FindMarkers(object = clr.obj, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1)))
+results.sct <- suppressWarnings(FindMarkers(object = sct.obj, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1)))
 
 test_that("Default settings work as expected", {
   expect_error(FindMarkers(object = pbmc_small))
@@ -29,6 +34,24 @@ test_that("Default settings work as expected", {
   expect_equal(markers.01[1, "p_val_adj"], 3.916481e-09)
   expect_equal(nrow(x = markers.01), 201)
   expect_equal(rownames(x = markers.01)[1], "TYMP")
+
+  # CLR normalization
+  expect_equal(results.clr[1, "p_val"], 1.209462e-11)
+  expect_equal(results.clr[1, "avg_logFC"], -0.8290693, tolerance = 1e-6)
+  expect_equal(results.clr[1, "pct.1"], 0.111)
+  expect_equal(results.clr[1, "pct.2"], 0.96)
+  expect_equal(results.clr[1, "p_val_adj"], 2.781762e-09)
+  expect_equal(nrow(x = results.clr), 85)
+  expect_equal(rownames(x = results.clr)[1], "S100A8")
+
+  # SCT normalization
+  expect_equal(results.sct[1, "p_val"], 6.225491e-11)
+  expect_equal(results.sct[1, "avg_logFC"], -0.6768721, tolerance = 1e-6)
+  expect_equal(results.sct[1, "pct.1"], 0.111)
+  expect_equal(results.sct[1, "pct.2"], 0.96)
+  expect_equal(results.sct[1, "p_val_adj"], 1.369608e-08)
+  expect_equal(nrow(x = results.sct), 92)
+  expect_equal(rownames(x = results.sct)[1], "TYMP")
 })
 
 
@@ -56,7 +79,7 @@ test_that("features parameter behaves correctly ", {
 
 results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = Cells(x = pbmc_small)[1:40], ident.2 = Cells(x = pbmc_small)[41:80], verbose = FALSE, base = exp(1)))
 test_that("passing cell names works", {
-  expect_equal(nrow(x = results), 176)
+  expect_equal(nrow(x = results), 190)
   expect_equal(results[1, "p_val"], 0.0001690882)
   expect_equal(results[1, "avg_logFC"], -1.790824, tolerance = 1e-6)
   expect_equal(results[1, "pct.1"], 0.075)
@@ -65,6 +88,28 @@ test_that("passing cell names works", {
   expect_equal(rownames(x = results)[1], "IFI30")
 })
 
+results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1), pseudocount.use = 0.1))
+results.clr <- suppressWarnings(FindMarkers(object = clr.obj, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1), pseudocount.use = 0.1))
+results.sct <- suppressWarnings(FindMarkers(object = sct.obj, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1), pseudocount.use = 0.1))
+test_that("setting pseudocount.use works", {
+  expect_equal(nrow(x = results), 202)
+  expect_equal(results[1, "avg_logFC"], -2.630395, tolerance = 1e-6)
+  expect_equal(nrow(x = results.clr), 182)
+  expect_equal(results.clr[1, "avg_logFC"], -2.317338, tolerance = 1e-6)
+  expect_equal(nrow(results.sct), 185)
+  expect_equal(results.sct[1, "avg_logFC"], -1.845681, tolerance = 1e-6)
+})
+
+results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1), mean.fxn = rowMeans))
+results.clr <- suppressWarnings(FindMarkers(object = clr.obj, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1), mean.fxn = rowMeans))
+results.sct <- suppressWarnings(FindMarkers(object = sct.obj, ident.1 = 0, ident.2 = 1, verbose = FALSE, base = exp(1), mean.fxn = rowMeans))
+test_that("setting mean.fxn works", {
+  expect_equal(nrow(x = results), 191)
+  expect_equal(results[1, "avg_logFC"], -4.204346, tolerance = 1e-6)
+  expect_equal(results.clr[1, "avg_logFC"], -1.353025, tolerance = 1e-6)
+  expect_equal(results.sct[1, "avg_logFC"], -1.064042, tolerance = 1e-6)
+})
+
 results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, logfc.threshold = 2, verbose = FALSE, base = exp(1)))
 test_that("logfc.threshold works", {
   expect_equal(nrow(x = results), 112)
@@ -78,18 +123,18 @@ test_that("logfc.threshold warns when none met", {
 
 results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, min.pct = 0.5, verbose = FALSE, base = exp(1)))
 test_that("min.pct works", {
-  expect_equal(nrow(x = results), 63)
+  expect_equal(nrow(x = results), 65)
   expect_gte(min(apply(X = results, MARGIN = 1, FUN = function(x) max(x[3], x[4]))), 0.5)
 })
 
-results <- expect_warning(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, min.pct = 1.0, verbose = FALSE, base = exp(1)))
+results <- expect_warning(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, min.pct = 2.0, verbose = FALSE, base = exp(1)))
 test_that("min.pct warns when none met", {
   expect_equal(nrow(x = results), 0)
 })
 
 results <- suppressWarnings(FindMarkers(object = pbmc_small, ident.1 = 0, ident.2 = 1, min.diff.pct = 0.5, verbose = FALSE, base = exp(1)))
 test_that("min.diff.pct works", {
-  expect_equal(nrow(x = results), 43)
+  expect_equal(nrow(x = results), 44)
   expect_gte(min(apply(X = results, MARGIN = 1, FUN = function(x) abs(x[4] - x[3]))), 0.5)
 })
 
@@ -150,7 +195,7 @@ Idents(object = t2) <- "groups"
 results2 <- suppressWarnings(FindMarkers(object = t2, ident.1 = "g1", ident.2 = "g2", verbose = FALSE, base = exp(1)))
 
 test_that("subset.ident works", {
-  expect_equal(nrow(x = results), 114)
+  expect_equal(nrow(x = results), 127)
   expect_equal(results, results2)
   expect_equal(results[1, "p_val"], 0.01293720)
   expect_equal(results[1, "avg_logFC"], 1.799280, tolerance = 1e-6)
@@ -236,6 +281,51 @@ test_that("LR test works", {
   expect_equal(rownames(x = results)[1], "LYZ")
 })
 
+# Tests for FindAllMarkers
+# -------------------------------------------------------------------------------
+results <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_small)))
+results.clr <- suppressMessages(suppressWarnings(FindAllMarkers(object = clr.obj)))
+results.sct <- suppressMessages(suppressWarnings(FindAllMarkers(object = sct.obj)))
+results.pseudo <- suppressMessages(suppressWarnings(FindAllMarkers(object = pbmc_small, pseudocount.use = 0.1)))
+
+test_that("FindAllMarkers works as expected", {
+  expect_equal(colnames(x = results), c("p_val", "avg_log2FC", "pct.1", "pct.2", "p_val_adj", "cluster", "gene"))
+  expect_equal(results[1, "p_val"], 9.572778e-13)
+  expect_equal(results[1, "avg_log2FC"], -5.820829, tolerance = 1e-6)
+  expect_equal(results[1, "pct.1"], 0.083)
+  expect_equal(results[1, "pct.2"], 0.909)
+  expect_equal(results[1, "p_val_adj"], 2.201739e-10)
+  expect_equal(nrow(x = results), 222)
+  expect_equal(rownames(results)[1], "HLA-DPB1")
+
+  # CLR normalization
+  expect_equal(results.clr[1, "p_val"], 1.209462e-11)
+  expect_equal(results.clr[1, "avg_log2FC"], -1.079924, tolerance = 1e-6)
+  expect_equal(results.clr[1, "pct.1"], 0.083)
+  expect_equal(results.clr[1, "pct.2"], 0.909)
+  expect_equal(results.clr[1, "p_val_adj"], 3.079373e-10)
+  expect_equal(nrow(x = results.clr), 200)
+  expect_equal(rownames(x = results.clr)[1], "HLA-DPB1")
+
+  # SCT normalization
+  expect_equal(results.sct[1, "p_val"], 6.225491e-11)
+  expect_equal(results.sct[1, "avg_log2FC"], -1.265307, tolerance = 1e-6)
+  expect_equal(results.sct[1, "pct.1"], 0.167)
+  expect_equal(results.sct[1, "pct.2"], 0.909)
+  expect_equal(results.sct[1, "p_val_adj"], 1.369608e-08)
+  expect_equal(nrow(x = results.sct), 201)
+  expect_equal(rownames(x = results.sct)[1], "HLA-DPB1")
+
+  # pseudocount.use = 0.1
+  expect_equal(results.pseudo[1, "p_val"], 9.572778e-13)
+  expect_equal(results.pseudo[1, "avg_log2FC"], -6.013818, tolerance = 1e-6)
+  expect_equal(results.pseudo[1, "pct.1"], 0.083)
+  expect_equal(results.pseudo[1, "pct.2"], 0.909)
+  expect_equal(results.pseudo[1, "p_val_adj"], 2.201739e-10)
+  expect_equal(nrow(x = results.pseudo), 222)
+  expect_equal(rownames(results.pseudo)[1], "HLA-DPB1")
+})
+
 # Tests for FindConservedMarkers
 # -------------------------------------------------------------------------------
 
@@ -261,7 +351,7 @@ if (requireNamespace('metap', quietly = TRUE)) {
     expect_equal(markers[1, "g1_p_val_adj"], 9.077279e-06)
     expect_equal(markers[1, "max_pval"], 4.983576e-05)
     expect_equal(markers[1, "minimump_p_val"], 7.893286e-08)
-    expect_equal(nrow(markers), 162)
+    expect_equal(nrow(markers), 179)
     expect_equal(rownames(markers)[1], "HLA-DRB1")
     expect_equal(markers[, "max_pval"], unname(obj = apply(X = markers, MARGIN = 1, FUN = function(x) max(x[c("g1_p_val", "g2_p_val")]))))
   })
@@ -286,7 +376,7 @@ if (requireNamespace('metap', quietly = TRUE)) {
     # expect_equal(markers.missing[1, "g2_pct.1"], 0.062)
     expect_equal(markers.missing[1, "g2_pct.2"], 0.95)
     expect_equal(markers.missing[1, "g2_p_val_adj"], 3.847695e-11)
-    expect_equal(nrow(markers.missing), 190)
+    expect_equal(nrow(markers.missing), 205)
     expect_equal(rownames(markers.missing)[1], "HLA-DPB1")
   })
 }
